package com.jpattern.orm.test;

import java.io.File;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map.Entry;

import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.junit.rules.TestName;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.jpattern.orm.JPOrm;
import com.jpattern.orm.test.db.DBData;
import com.jpattern.shared.util.Chronometer;

/**
 * 
 * @author Francesco Cina
 * 
 *         20/mag/2011
 */
@RunWith(Parameterized.class)
//BaseTestAllDB
public abstract class BaseTestAllDB {

	public static ApplicationContext CONTEXT = null;

	private final String TEST_FILE_INPUT_BASE_PATH = "./src/test/files"; //$NON-NLS-1$
	private final String TEST_FILE_OUTPUT_BASE_PATH = "./target/test/files"; //$NON-NLS-1$
	private final TestData testData;

	public BaseTestAllDB(final String testName, final TestData testData) {
		this.testData = testData;
	}

	@Parameterized.Parameters(name="{0}")
	public static Collection<Object[]> generateData() {
		if (CONTEXT == null) {
			CONTEXT = new ClassPathXmlApplicationContext("spring-context.xml"); //$NON-NLS-1$
		}

		List<Object[]> parameters = new ArrayList<Object[]>();
		for ( Entry<String, DBData> dbDataEntry :  CONTEXT.getBeansOfType(DBData.class).entrySet() ) {
			DBData dbData = dbDataEntry.getValue();
			if ( dbData.isDbAvailable() ) {
				parameters.add(new Object[]{ dbData.getDBType() + "_DataSource", new TestData(dbData.getDataSourceSessionProvider(), dbData.getDataSource(), dbData.getDBType(), dbData.supportMultipleSchemas()) }); //$NON-NLS-1$
				parameters.add(new Object[]{ dbData.getDBType() + "_JdbcTemplate", new TestData(dbData.getJdbcTemplateSessionProvider(), dbData.getDataSource(), dbData.getDBType(), dbData.supportMultipleSchemas()) }); //$NON-NLS-1$

			}
		}
		return parameters;
	}

	@Rule
	public final TestName name = new TestName();
	private final Chronometer chronometer = new Chronometer();
	private final Logger logger = LoggerFactory.getLogger(this.getClass());

	@Before
	public void setUpBeforeTest() {

		chronometer.restart();

		getLogger().info("==================================================================="); //$NON-NLS-1$
		getLogger().info("BEGIN TEST " + name.getMethodName()); //$NON-NLS-1$
		getLogger().info("==================================================================="); //$NON-NLS-1$

	}

	@After
	public void tearDownAfterTest() {

		chronometer.pause();
		final String time = new BigDecimal(chronometer.read()).divide(
				new BigDecimal(1000)).toString();

		getLogger().info("==================================================================="); //$NON-NLS-1$
		getLogger().info("END TEST " + name.getMethodName()); //$NON-NLS-1$
		getLogger().info("Execution time: " + time + " seconds"); //$NON-NLS-1$ //$NON-NLS-2$
		getLogger().info("==================================================================="); //$NON-NLS-1$

	}

	protected String getTestInputBasePath() {
		return TEST_FILE_INPUT_BASE_PATH;
	}

	protected String getTestOutputBasePath() {
		mkDir(TEST_FILE_OUTPUT_BASE_PATH);
		return TEST_FILE_OUTPUT_BASE_PATH;
	}

	protected void mkDir(final String dirPath) {
		final File path = new File(dirPath);
		if (!path.exists()) {
			path.mkdirs();
		}
	}

	protected JPOrm getJPOrm() {
		return new JPOrm(testData.getSessionProvider());
	}

	public TestData getTestData() {
		return testData;
	}

	public Logger getLogger() {
		return logger;
	}

}
