package com.jpattern.orm.test.db;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;
import org.springframework.transaction.PlatformTransactionManager;

import com.jpattern.orm.JPO;
import com.jpattern.orm.JPOrm;
import com.jpattern.orm.dialect.DBType;
import com.jpattern.orm.script.ScriptExecutor;
import com.jpattern.orm.session.Session;
import com.jpattern.orm.session.SessionProvider;
import com.jpattern.orm.session.TransactionCallback;
import com.jpattern.orm.session.datasource.DataSourceSessionProvider;
import com.jpattern.orm.session.jdbctemplate.JdbcTemplateSessionProvider;

@Component
@Lazy
public class DerbyDBData implements DBData {


	@Resource(name = "derbyDataSource")
	private DataSource dataSource;
	@Resource(name = "derbyTransactionManager")
	public PlatformTransactionManager transactionManager;
	@Value("${derby.isDbAvailable}")
	private boolean isDbAvailable;
	@Value("${derby.supportMultipleSchemas}")
	private boolean supportMultipleSchemas;

	@Override
	public DBType getDBType() {
		return DBType.DERBY;
	}

	@Override
	public boolean isDbAvailable() {
		return isDbAvailable;
	}

	@Override
	public SessionProvider getJdbcTemplateSessionProvider() {
		return new JdbcTemplateSessionProvider(getDataSource(), transactionManager);
	}

	@Override
	public SessionProvider getDataSourceSessionProvider() {
		return new DataSourceSessionProvider(getDataSource());
	}

	@Override
	public boolean supportMultipleSchemas() {
		return supportMultipleSchemas;
	}

	@Override
	public DataSource getDataSource() {
		return dataSource;
	}

	@PostConstruct
	private void init() {
		final JPO jpOrm = new JPOrm(getDataSourceSessionProvider());

		jpOrm.session().doInTransaction(new TransactionCallback<Void>() {

			@Override
			public Void doInTransaction(final Session session) {
				final ScriptExecutor scriptExecutor = session.scriptExecutor();
				try {
					scriptExecutor.execute(getClass().getResourceAsStream("/sql/derby_create_db.sql")); //$NON-NLS-1$
				} catch (Exception e) {
					throw new RuntimeException(e);
				}
				return null;
			}

		});

	}

}
