package com.jpattern.orm.test.db;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;
import org.springframework.transaction.PlatformTransactionManager;

import com.jpattern.orm.dialect.DBType;
import com.jpattern.orm.session.SessionProvider;
import com.jpattern.orm.session.datasource.DataSourceSessionProvider;
import com.jpattern.orm.session.jdbctemplate.JdbcTemplateSessionProvider;

@Component
@Lazy
public class HsqldbDBData implements DBData {

	@Resource(name = "hsqldbDataSource")
	private DataSource dataSource;
	@Resource(name = "hsqldbTransactionManager")
	public PlatformTransactionManager transactionManager;
	@Value("${hsqldb.isDbAvailable}")
	private boolean isDbAvailable;
	@Value("${hsqldb.supportMultipleSchemas}")
	private boolean supportMultipleSchemas;

	@Override
	public boolean isDbAvailable() {
		return isDbAvailable;
	}

	@Override
	public SessionProvider getJdbcTemplateSessionProvider() {
		return new JdbcTemplateSessionProvider(dataSource, transactionManager);
	}

	@Override
	public SessionProvider getDataSourceSessionProvider() {
		return new DataSourceSessionProvider(dataSource);
	}

	@Override
	public DBType getDBType() {
		return DBType.HSQLDB;
	}

	@Override
	public boolean supportMultipleSchemas() {
		return supportMultipleSchemas;
	}

	@Override
	public DataSource getDataSource() {
		return dataSource;
	}

}
